/////////////////////////////////////////
//      Nebula M42 by Polecatsoft      //
/////////////////////////////////////////
/* Description:                        
   These class can be used to read and write data
   from and to *.ini files.
///////////////////////////////////////*/

#ifndef iniReader_h
#define iniReader_h

#include <string>
#include <vector>

struct iniEntry
{
	std::string name, value;
	int category;
};

class iniReader
{
private:
	std::vector<iniEntry> entries;
	std::vector<std::string> categories;
	std::vector<std::vector<iniEntry>::iterator> firstCategoryEntry;
	int curCategory;
	std::vector<iniEntry>::iterator curEntry;
	std::string *argv;
public:
	//Constructor sets category to -1 (-1 is at global scope)
	iniReader();
	//Destructor does nothing
	~iniReader();
	//This will load all the values from the file filename into memory
	//Returns false if the file could not be opened
	bool load(const std::string filename);
	//This will set the category to newCategory, or to the global category if
	//newCategory is not found
	//Returns < 0 if the category wasn't found or >= 0 if it was found
	int setCategory(const std::string newCategory);
	//Sets the global category as the current category
	void setGlobalCategory();
	//Returns the value of name in the current category
	std::string getValue(const std::string name,
		const std::string defaultValue = "");
	//Returns an integer version of the value
	int getIValue(const std::string name, const int defaultValue);
	//Returns a floating point version of the value
	float getFValue(const std::string name, const float defaultValue);
	//Returns a boolean version of the value
	bool getBValue(const std::string name, const bool defaultValue);
	//Returns an array of strings. It extracts values from the value the entry
	//name, that are seperated with seperator. values is set to the number of
	//entries returned. The returned array will be deleted when this function
	//is called again or if cleanUpArgv() is called
	//Returns 0 on error.
	//Warning!!! Do not delete the return value on your own!
	std::string *getArgvValue(const std::string name, int &values,
		const char seperator = ',');
	//Clean up the memory returned by getArgvValue()
	void cleanUpArgv();
	//Returns the number of entries loaded
	int getNumberOfEntries();
	//Returns the name of the current category
	//Returns an empty string if curcategory is global
	std::string getCurCategory();
	//Returns the name of the next category
	//Returns an empty string if no further categories exist
	std::string getNextCategory();
	//Resets the curEntry to the first entry of the current category
	void resetEntryPointer();
	//Returns the next iniEntry of the current category or 0 on error
	iniEntry *getNextEntry();
};

class iniWriter
{
private:
	std::vector<iniEntry> entries;
	std::vector<std::string> categories;
	int curCategory;
public:
	//Constructor sets category to -1 (-1 is at global scope)
	iniWriter();
	//Destructor does nothing
	inline ~iniWriter() {}
	//Add a category to the list
	void addCategory(const std::string name);
	//This will set the category to newCategory, or to the global category if
	//newCategory is not found
	void setCategory(const std::string newCategory);
	//Sets the global category as the current category
	void setGlobalCategory();
	//Adds a value to the active category
	void addEntry(const std::string name, const std::string value);
	void addEntry(const std::string name, const int value);
	void addEntry(const std::string name, const float value);
	void addEntry(const std::string name, const bool value);
	//This will save all the values from memory into the file filename
	void save(const std::string filename);
	//Adds all entries from reader to this iniWriter instance
	void copyAll(iniReader &reader);
};

#endif
